/*
* 07/08/2003 - 19:09:03
*
* TSTApplet.java - 
* Copyright (C) 2003 Ashwin Jayaprakash
* ashwinjay@rediffmail.com
* www.javaforu.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package searchassist.ui;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.ObjectInputStream;
import java.util.List;

import javax.swing.JApplet;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JSeparator;

import netscape.javascript.JSObject;
import com.wallyflint.util.TernarySearchTree;

import com.mattwelsh.appframework.ui.AutoCompleteDictionary;
import com.mattwelsh.appframework.ui.AutoCompleteTextField;

/**
 * @author Ashwin Jayaprakash
 */
public class TSTApplet extends JApplet
{
    public static final String PARAM_SERIALIZED_TST_FILE = "SerializedTSTFile";
    public static final String PARAM_ID_RENDERER_IMPL = "IDRendererImpl";
    private static final String TOOL_TIP =
        "Enter search text above \n"
            + "and press Enter. \n"
            + "If the word is there \n"
            + "in the database, you \n"
            + "will be prompted with \n"
            + "the nearest match. \n"
            + "If nothing appears, \n"
            + "in the list below, then \n"
            + "the word you typed \n"
            + "does not exist in the \n"
            + "database.";

    private JPanel textPanel;
    private JSeparator separator;
    private AutoCompleteTextField textField;
    private javax.swing.JTextArea textArea;
    private javax.swing.JScrollPane textAreaScrlPane;
    private javax.swing.JPanel leftPanel;
    private javax.swing.JSplitPane splitPane;
    private javax.swing.JComponent renderingComponent;

    private TernarySearchTree tst;
    private IDRenderer idRenderer;
    private AutoCompleteDictionary dictionary;

    public void init()
    {
        super.init();

        final String serTSTFile = getParameter(PARAM_SERIALIZED_TST_FILE);
        final String idRendererImpl = getParameter(PARAM_ID_RENDERER_IMPL);
        try
        {
            ObjectInputStream ois = new ObjectInputStream(getClass().getResourceAsStream(serTSTFile));
            tst = (TernarySearchTree) ois.readObject();

            Class clazz = getClass().getClassLoader().loadClass(idRendererImpl);
            idRenderer = (IDRenderer) clazz.newInstance();

            //Anonymous inner class implementing the Private Interface pattern.
            Attributes attrs = new Attributes()
            {
                public String getParameter(String paramName)
                {
                    return TSTApplet.this.getParameter(paramName);
                }
                
                public JSObject getWindow()
                {
                    return JSObject.getWindow(TSTApplet.this);
                }
            };
            idRenderer.init(attrs);
            renderingComponent = idRenderer.getRenderingComponent();
        } //end try
        catch (Throwable t)
        {
            t.printStackTrace();
            throw new RuntimeException("Error occured while initializing resources. Stopping Applet.", t);
        } //end catch

        initComponents();

        dictionary = new TSTDictionary(tst, textField, textArea);
        textField.setDictionary(dictionary);
    } //end constr

    private void initComponents()
    {
        splitPane = new javax.swing.JSplitPane();
        leftPanel = new javax.swing.JPanel();

        textPanel = new javax.swing.JPanel();
        textPanel.setLayout(new java.awt.BorderLayout());
        separator = new javax.swing.JSeparator();
        separator.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.RAISED));
        textPanel.add(separator, java.awt.BorderLayout.NORTH);

        //Add null for now.
        textField = new AutoCompleteTextField(null);
        textPanel.add(textField, java.awt.BorderLayout.CENTER);
        textAreaScrlPane = new javax.swing.JScrollPane();
        textArea = new javax.swing.JTextArea();

        getContentPane().setLayout(new java.awt.GridLayout(1, 1));

        setName("TSTApplet");
        splitPane.setBorder(null);
        splitPane.setDividerSize(1);
        splitPane.setAutoscrolls(true);
        leftPanel.setLayout(new java.awt.BorderLayout());

        leftPanel.setPreferredSize(new java.awt.Dimension(160, 400));
        textField.setFont(new java.awt.Font("Arial", 1, 12));
        textField.setText("");
        textField.setToolTipText(TOOL_TIP);
        textField.setBorder(null);
        textField.setCaretColor(new java.awt.Color(255, 124, 12));
        textField.setSelectionColor(new java.awt.Color(102, 204, 0));
        textField.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                textFieldActionPerformed(evt);
            }
        });

        leftPanel.add(textPanel, java.awt.BorderLayout.NORTH);

        textAreaScrlPane.setBorder(null);
        textArea.setEditable(false);
        textArea.setFont(new java.awt.Font("Arial", 2, 12));
        textArea.setForeground(new java.awt.Color(102, 102, 102));
        textArea.setRows(7);
        textArea.setText(TOOL_TIP);
        textArea.setBorder(null);
        textAreaScrlPane.setViewportView(textArea);

        leftPanel.add(textAreaScrlPane, java.awt.BorderLayout.CENTER);

        splitPane.setLeftComponent(leftPanel);

        splitPane.setRightComponent(renderingComponent);

        getContentPane().add(splitPane);
    }

    private void textFieldActionPerformed(java.awt.event.ActionEvent evt)
    {
        String str = textField.getText();
        if (str.equals(""))
        {
            return;
        } //end if

        List list = tst.get(str);
        if (list == null)
        {
            return;
        } //end if

        idRenderer.renderNodes(list);
    } //end method

    public static void main(String args[])
    {
        DemoFrame app = new DemoFrame("Ternary Search Tree");
        app.setSize(500, 600);
        app.show();
    } //end main

    private static class DemoFrame extends JFrame
    {
        public DemoFrame(String frameTitle)
        {
            super(frameTitle);

            JApplet applet = new TSTApplet();
            applet.init();
            applet.start();
            getContentPane().add(applet, java.awt.BorderLayout.CENTER);

            addWindowListener(new WindowAdapter()
            {
                public void windowClosing(WindowEvent event)
                {
                    dispose();
                    System.exit(0);
                }
            });
        }
    } //end inner class DemoFrame
} //end class
