/*
* 07/08/2003 - 19:09:03
*
* MozillaBookmark2XMLConverter.java - 
* Original source: http://java.sun.com/xml/jaxp/dist/1.1/docs/tutorial/sax/work/Echo12.java
* Copyright (C) 2003 Ashwin Jayaprakash
* ashwinjay@rediffmail.com
* www.javaforu.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package searchassist.utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.helpers.DefaultHandler;

/**
 * @author Ashwin Jayaprakash
 */
public class MozillaBookmark2XMLConverter extends DefaultHandler implements LexicalHandler
{
	private Writer out;
	private String indentString = "   "; // Amount to indent
	private int indentLevel = 0;
	private String lineEnd = System.getProperty("line.separator");

	private StringBuffer strBuffer = new StringBuffer();
	private String tagName;
	private String recentlySeenH3Value = null;
    private String aTitle = null;
	private String aValue = null;

	private String recentlySeenEvent = null;
	private static final String EVT_START_ELEM = "startElement";
	private static final String EVT_CHARACTERS = "characters";
	private static final String EVT_END_ELEM = "endElement";

	private static final boolean DEBUG = true;

	public MozillaBookmark2XMLConverter(Writer outWriter)
	{
		out = outWriter;
	} //end constr

	//===========================================================
	// SAX DocumentHandler methods
	//===========================================================

	public void setDocumentLocator(Locator l)
	{
		// Save this to resolve relative URIs or to give diagnostics.
		try
		{
			out.flush();
		}
		catch (IOException e)
		{
			// Ignore errors
		}
	}

	public void startDocument() throws SAXException
	{
        emit("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		nl();
	}

	public void startElement(String namespaceURI, String lName, // local name
	String qName, // qualified name
	Attributes attrs) throws SAXException
	{
		recentlySeenEvent = EVT_START_ELEM;

		tagName = lName; // element name
		if ("".equals(tagName))
		{
			tagName = qName; // namespaceAware = false
		} //end if

		if (tagName.equalsIgnoreCase("dl"))
		{
			indentLevel++;
			nl();
			emit("<category>");

			indentLevel++;
			nl();
			emit("<title>");
			if (recentlySeenH3Value != null)
			{
				nl();
				emit(recentlySeenH3Value);

				if (DEBUG)
				{
					System.out.println("recentlySeenH3Value:: " + recentlySeenH3Value);
				} //end inner if

				recentlySeenH3Value = null;
			} //end inner if
			nl();
			emit("</title>");
			indentLevel--;
		} //end if      

		if (tagName.equalsIgnoreCase("a"))
		{
			aValue = attrs.getValue("HREF");

			indentLevel++;
			nl();
			emit("<item>");
		} //end if
	} //end method

	public void characters(char buf[], int offset, int len) throws SAXException
	{
		String s = new String(buf, offset, len).trim();
		if (s.equals(""))
		{
			recentlySeenEvent = EVT_CHARACTERS;
			return;
		} //end if

		if (DEBUG)
		{
			System.out.println("char tagName: " + tagName);
			System.out.println("s:: " + s);
		}

		if (tagName.equalsIgnoreCase("h3"))
		{
			//Just append this to the string.
			if (recentlySeenEvent == EVT_CHARACTERS)
			{
				recentlySeenH3Value += " " + s;
			} //end if
			//Else, this is a new tag.
			else
			{
				recentlySeenH3Value = s;
			} //end else

			if (DEBUG)
			{
				System.out.println("setting recentlySeenH3Value:: " + recentlySeenH3Value);
			} //end if
		} //end if

		if (tagName.equalsIgnoreCase("a"))
		{
            //Just append this to the string.
            if (recentlySeenEvent == EVT_CHARACTERS)
            {
                aTitle += " " + s;
            } //end if
            //Else, this is a new tag.
            else
            {
                aTitle = s;
            } //end else
		} //end if

		recentlySeenEvent = EVT_CHARACTERS;
	} //end method

	public void endElement(String namespaceURI, String sName, // simple name
	String qName // qualified name
	) throws SAXException
	{
		recentlySeenEvent = EVT_END_ELEM;

		tagName = sName; // element name
		if ("".equals(tagName))
		{
			tagName = qName; // namespaceAware = false
		} //end if

		if (tagName.equalsIgnoreCase("a"))
		{
            indentLevel++;
            nl();
            emit("<title>");
            nl();
            emit(aTitle);
            nl();
            emit("</title>");
            nl();
            emit("<link>");
            nl();
            emit(aValue);
            nl();
            emit("</link>");
            indentLevel--;
           
			nl();
			emit("</item>");
			indentLevel--;
		} //end if

		if (tagName.equalsIgnoreCase("dl"))
		{
			nl();
			emit("</category>");
			indentLevel--;
		} //end if
	}

	public void ignorableWhitespace(char buf[], int offset, int len) throws SAXException
	{
		// Ignore it
	}

	public void processingInstruction(String target, String data) throws SAXException
	{
		nl();
		emit("<?" + target + " " + data + "?>");
	}

	public void endDocument() throws SAXException
	{
		try
		{
			nl();
			out.flush();
		}
		catch (IOException e)
		{
			throw new SAXException("I/O error", e);
		}
	}

	//===========================================================
	// SAX ErrorHandler methods
	//===========================================================

	// treat validation errors as fatal
	public void error(SAXParseException e) throws SAXParseException
	{
		throw e;
	}

	// dump warnings too
	public void warning(SAXParseException err) throws SAXParseException
	{
		System.out.println("** Warning" + ", line " + err.getLineNumber() + ", uri " + err.getSystemId());
		System.out.println("   " + err.getMessage());
	}

	//===========================================================
	// LexicalEventListener methods
	//===========================================================

	public void comment(char[] ch, int start, int length) throws SAXException
	{
	}

	public void startCDATA() throws SAXException
	{
	}

	public void endCDATA() throws SAXException
	{
	}

	public void startEntity(java.lang.String name) throws SAXException
	{
	}

	public void endEntity(java.lang.String name) throws SAXException
	{
	}

	public void startDTD(String name, String publicId, String systemId) throws SAXException
	{
	}

	public void endDTD() throws SAXException
	{
	}

	protected boolean addUniqueIDForTag(String tagName)
	{
		boolean retValue = false;
		if (tagName.equals("category") || tagName.equals("item"))
		{
			retValue = true;
		} //end if

		return retValue;
	} //end method

	protected String getUniqueIDAttrName()
	{
		return "uniqueID";
	} //end method

	//===========================================================
	// Utility Methods ...
	//===========================================================

	// Wrap I/O exceptions in SAX exceptions, to
	// suit handler signature requirements
	private void emit(String s) throws SAXException
	{
		try
		{
			out.write(s);
			out.flush();
		}
		catch (IOException e)
		{
			throw new SAXException("I/O error", e);
		}
	}

	// Start a new line
	// and indent the next line appropriately
	private void nl() throws SAXException
	{
		try
		{
			out.write(lineEnd);
			for (int i = 0; i < indentLevel; i++)
				out.write(indentString);
		}
		catch (IOException e)
		{
			throw new SAXException("I/O error", e);
		}
	}

	public static void main(String args[])
	{
		if (args.length < 2)
		{
			System.out.println("Usage: cmd mozilla_bookmarks_input_file xml_output_file");
			System.out.println("Note: The input html document must have complete tags like - <P/> instead of <P>,");
			System.out.println(
				"      <BR/> instead of <BR>, <HR/> instead of <HR>, <DT/> instead of <DT> and no single &.");
			System.out.println("      &amp; or &nbsp; are allowed, but not & in queries.");
			System.out.println("      Also the document be a well formed xml document with <?xml version=\"1.0\"?>");
			System.out.println("      and single root element");
			System.exit(1);
		}

		try
		{
			DefaultHandler handler = new MozillaBookmark2XMLConverter(new FileWriter(new File(args[1])));
			SAXParserFactory factory = SAXParserFactory.newInstance();
			factory.setValidating(false);
			SAXParser parser = factory.newSAXParser();
			XMLReader xmlReader = parser.getXMLReader();
			xmlReader.setProperty("http://xml.org/sax/properties/lexical-handler", handler);
			parser.parse(new FileInputStream(args[0]), handler);
		}
		catch (SAXParseException spe)
		{
			// Error generated by the parser
			System.out.println("\n** Parsing error" + ", line " + spe.getLineNumber() + ", uri " + spe.getSystemId());
			System.out.println("   " + spe.getMessage());
			// Use the contained exception, if any
			Exception x = spe;
			if (spe.getException() != null)
			{
				x = spe.getException();
			} //end if
			x.printStackTrace();
		}
		catch (SAXException sxe)
		{
			// Error generated by this application
			// (or a parser-initialization error)
			Exception x = sxe;
			if (sxe.getException() != null)
			{
				x = sxe.getException();
			} //end if
			x.printStackTrace();
		}
		catch (ParserConfigurationException pce)
		{ // Parser with specified options can't be built
			pce.printStackTrace();
		}
		catch (IOException ioe)
		{ // I/O error
			ioe.printStackTrace();
		}

		System.exit(0);

	} //end main

} //end class
