/*
* 07/08/2003 - 19:09:03
*
* SpecialCharactersFilter.java - 
* Copyright (C) 2003 Ashwin Jayaprakash
* ashwinjay@rediffmail.com
* www.javaforu.com
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package searchassist.utils;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;

/**
 * @author Ashwin Jayaprakash
 */
public class SpecialCharactersFilter
{
	public static final String[][] replacements = { { "<P>", "<P />" }, {
			"<p>", "<p />" }, {
			"<HR>", "<HR />" }, {
			"<BR>", "<BR />" }, {
			"<DT>", "<DT />" }, {
			"'", "`" }
	};

	private static final boolean DEBUG = true;

	public static void filter(String[] args, boolean filter)
	{
		final int from = (filter ? 0 : 1);
		final int to = (filter ? 1 : 0);

		BufferedReader bufferedReader = null;
		FileWriter fileWriter = null;

		try
		{
			FileReader fileReader = new FileReader(new File(args[0]));
			bufferedReader = new BufferedReader(fileReader);
			fileWriter = new FileWriter(new File(args[1]));

			String str = null;
			do
			{
				str = bufferedReader.readLine();
				if (DEBUG)
				{
					System.out.println("read str=" + str);
				} //end if

				if (str == null)
				{
					break;
				} //end if

				//From http://www.hivelogic.com/urlcleaner.php
				int ampersandIdx = 0;
				final String and = " &amp; ";
				final String lonelyAnd = "&#038;";

				while (ampersandIdx > -1 && ampersandIdx < str.length())
				{
					ampersandIdx = str.indexOf('&', ampersandIdx);
					if (ampersandIdx > -1)
					{
						if ((ampersandIdx > 0 && ampersandIdx < str.length())
							&& (str.charAt(ampersandIdx - 1) == ' ' && str.charAt(ampersandIdx + 1) == ' '))
						{
							StringBuffer stringBuffer = new StringBuffer(str);
							stringBuffer.replace(ampersandIdx, ampersandIdx + 1, and);
							str = stringBuffer.toString();
							ampersandIdx += and.length();

							if (DEBUG)
							{
								System.out.println("anded str=" + str);
							} //end if
						} //end inner if
						else
						{
							int ctr = 0;
							int i = ampersandIdx + 1;
							int len = str.length();
							boolean isEntity = false;
							for (; i < len; i++)
							{
								char c = str.charAt(i);
								ctr++;

								if (DEBUG)
								{
									System.out.print("i=" + i + ", c=" + c + "\n");
								} //end if

								//Valid chars in entity.                               
								if (Character.isLetterOrDigit(c) || c == '#' || c == '_' || c == '.' || c == '-')
								{
									if (DEBUG)
									{
										System.out.println("--continue");
									} //end if

									//First char after & must be letter or digit.
									if (ctr == 1 && !Character.isLetterOrDigit(c))
									{
										break;
									} //end if

									continue;
								} //end innermost if
								else if (c == ';')
								{
									//Ok, this is an XML Entity. So, it's alright.
									ampersandIdx += ctr;
									isEntity = true;
									break;
								} //end innermost else if
								else
								{
									break;
								} //end innermost else
							} //end for

							//If entity was not found and reached end of string.
							if (!isEntity)
							{
								StringBuffer stringBuffer = new StringBuffer(str);
								stringBuffer.replace(ampersandIdx, ampersandIdx + 1, lonelyAnd);
								str = stringBuffer.toString();
								ampersandIdx += lonelyAnd.length();
							} //end if
						} //end inner else
					} //end if
				} //end while

				for (int i = 0; i < replacements.length; i++)
				{
					if (DEBUG)
					{
						System.out.println("i=" + i);
						System.out.println("str=" + str);
					} //end if
					str = str.replaceAll(replacements[i][from], replacements[i][to]);
				} //end for
				fileWriter.write(str);
				fileWriter.write("\n");
			}
			while (true);
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			try
			{
				if (bufferedReader != null)
				{
					bufferedReader.close();
				} //end if
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}

			try
			{
				if (fileWriter != null)
				{
					fileWriter.close();
				} //end if
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		} //end finally
	} //end method
	public static void main(String args[])
	{
		final String usage = "Usage: cmd input_file output_file true/false";
		if (args.length < 3)
		{
			System.out.println(usage);
			System.exit(1);
		} //end if
		if (args[2].equalsIgnoreCase("true"))
		{
			filter(args, true);
		} //end if
		else if (args[2].equalsIgnoreCase("false"))
		{
			filter(args, false);
		} //end else if
		else
		{
			System.out.println(usage);
			System.exit(1);
		} //end else
		System.exit(0);
	} //end main
} //end class
